package dev.pfaff.altargui.widgets;

import dev.pfaff.altargui.*;
import dev.pfaff.altargui.element.CustomLayoutSingleChildWidgetInstance;
import dev.pfaff.altargui.internal.util.StencilLayoutContext;
import org.jetbrains.annotations.NotNull;

/**
 * A widget that calls the provided {@code builder} every layout.
 */
public record LayoutBuilderWidget(Builder builder) implements Widget {
	@Override
	public @NotNull WidgetInstance<LayoutBuilderWidget> instantiate(WidgetInstantiationArgs args) {
		return new State(args);
	}

	@FunctionalInterface
	public interface Builder {
		Widget build(float width, float height);
	}

	private static final class State extends CustomLayoutSingleChildWidgetInstance<LayoutBuilderWidget> {
		private Builder builder;

		private State(WidgetInstantiationArgs args) {
			super(args);
		}

		@Override
		public void configure(StateContext stateContext, LayoutBuilderWidget widget, boolean init) {
			if (builder != widget.builder) {
				builder = widget.builder;
				markNeedsLayout();
			}
		}

		@Override
		protected void computeStencilLayout(StateContext context) {
			// TODO: add a profiling counter for the number of computeLayout calls with unchanged allocated rect
			// TODO: cache result of builder.build if profiling counter suggests it would be beneficial
			setChild(context, builder.build(allocatedWidth(), allocatedHeight()), false);
			StencilLayoutContext.updateMountAndLayout(context, this, getChildElement(), (e, ctx) -> {
				e.copyAvailableSizeFrom(this);
				e.stencilLayout(ctx);
				copyComputedSizeFrom(e);
			});
		}

		@Override
		protected void computeTraceLayout() {
			var e = getChildElement();
			e.copyPositionFrom(this);
			e.traceLayout();
		}
	}
}
